@@ -1,8 +1,9 @@
 package Dancer2::Core::Role::SessionFactory;
-#ABSTRACT: Role for session factories
-$Dancer2::Core::Role::SessionFactory::VERSION = '0.152000';
-use strict;
-use warnings;
+# ABSTRACT: Role for session factories
+$Dancer2::Core::Role::SessionFactory::VERSION = '0.166001';
+use Moo::Role;
+with 'Dancer2::Core::Role::Engine';
+
 use Carp 'croak';
 use Class::Load 'try_load_class';
 use Dancer2::Core::Session;
@@ -11,9 +12,7 @@ use Digest::SHA 'sha1';
 use List::Util 'shuffle';
 use MIME::Base64 'encode_base64url';
 
-use Moo::Role;
-with 'Dancer2::Core::Role::Engine';
-
+sub hook_aliases { +{} }
 sub supported_hooks {
     qw/
       engine.session.before_retrieve
@@ -34,6 +33,12 @@ sub _build_type {
     'SessionFactory';
 }    # XXX vs 'Session'?  Unused, so I can't tell -- xdg
 
+has log_cb => (
+    is      => 'ro',
+    isa     => CodeRef,
+    default => sub { sub {1} },
+);
+
 has cookie_name => (
     is      => 'ro',
     isa     => Str,
@@ -254,7 +259,7 @@ Dancer2::Core::Role::SessionFactory - Role for session factories
 
 =head1 VERSION
 
-version 0.152000
+version 0.166001
 
 =head1 DESCRIPTION
 
@@ -438,7 +443,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
