@@ -1,28 +1,35 @@
 # ABSTRACT: Role for template engines
 
 package Dancer2::Core::Role::Template;
-$Dancer2::Core::Role::Template::VERSION = '0.152000';
+$Dancer2::Core::Role::Template::VERSION = '0.166001';
 use Dancer2::Core::Types;
 use Dancer2::FileUtils qw'path';
 use Carp 'croak';
 
-use Data::Dumper;
 use Moo::Role;
 with 'Dancer2::Core::Role::Engine';
 
-sub supported_hooks {
-    qw/
-      engine.template.before_render
-      engine.template.after_render
-      engine.template.before_layout_render
-      engine.template.after_layout_render
-      /;
+sub hook_aliases {
+    {
+        before_template_render => 'engine.template.before_render',
+        after_template_render  => 'engine.template.after_render',
+        before_layout_render   => 'engine.template.before_layout_render',
+        after_layout_render    => 'engine.template.after_layout_render',
+    }
 }
 
+sub supported_hooks { values %{ shift->hook_aliases } }
+
 sub _build_type {'Template'}
 
 requires 'render';
 
+has log_cb => (
+    is      => 'ro',
+    isa     => CodeRef,
+    default => sub { sub {1} },
+);
+
 has name => (
     is      => 'ro',
     lazy    => 1,
@@ -71,6 +78,12 @@ has settings => (
     writer  => 'set_settings',
 );
 
+has layout_dir => (
+    is      => 'ro',
+    isa     => Str,
+    default => sub {'layouts'},
+);
+
 sub _template_name {
     my ( $self, $view ) = @_;
     my $def_tmpl_ext = $self->default_tmpl_ext();
@@ -87,8 +100,17 @@ sub view_pathname {
 
 sub layout_pathname {
     my ( $self, $layout ) = @_;
-    $layout = $self->_template_name($layout);
-    return path( $self->views, 'layouts', $layout );
+
+    return path(
+        $self->views,
+        $self->layout_dir,
+        $self->_template_name($layout),
+    );
+}
+
+sub pathname_exists {
+    my ( $self, $pathname ) = @_;
+    return -f $pathname;
 }
 
 sub render_layout {
@@ -154,7 +176,7 @@ sub _prepare_tokens_options {
 
     # these are the default tokens provided for template processing
     $tokens ||= {};
-    $tokens->{perl_version}   = $];
+    $tokens->{perl_version}   = $^V;
     $tokens->{dancer_version} = Dancer2->VERSION;
 
     $tokens->{settings} = $self->settings;
@@ -207,7 +229,7 @@ Dancer2::Core::Role::Template - Role for template engines
 
 =head1 VERSION
 
-version 0.152000
+version 0.166001
 
 =head1 DESCRIPTION
 
@@ -226,6 +248,43 @@ In order to implement this role, the consumer B<must> implement the method C<ren
 
 =back
 
+Any template receives the following tokens, by default:
+
+=over 4
+
+=item * C<perl_version>
+
+Current version of perl, effectively C<$^V>.
+
+=item * C<dancer_version>
+
+Current version of Dancer2, effectively C<<Dancer2->VERSION>>.
+
+=item * C<settings>
+
+A hash of the application configuration.
+
+=item * C<request>
+
+The current request object.
+
+=item * C<params>
+
+A hash reference of all the parameters.
+
+Currently the equivalent of C<< $request->params >>.
+
+=item * C<vars>
+
+The list of request variables, which is what you would get if you
+called the C<vars> keyword.
+
+=item * C<session>
+
+The current session data, if a session exists.
+
+=back
+
 =head1 ATTRIBUTES
 
 =head2 name
@@ -248,6 +307,12 @@ Path to the directory containing the views.
 
 Path to the directory containing the layouts.
 
+=head2 layout_dir
+
+Relative path to the layout directory.
+
+Default: B<layouts>.
+
 =head2 engine
 
 Contains the engine.
@@ -262,6 +327,14 @@ Returns the full path to the requested view.
 
 Returns the full path to the requested layout.
 
+=head2 pathname_exists($pathname)
+
+Returns true if the requested pathname exists. Can be used for either views
+or layouts:
+
+    $self->pathname_exists( $self->view_pathname( 'some_view' ) );
+    $self->pathname_exists( $self->layout_pathname( 'some_layout' ) );
+
 =head2 render_layout($layout, \%tokens, \$content)
 
 Render the layout with the applied tokens
@@ -280,7 +353,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
