@@ -1,12 +1,14 @@
 package Dancer2::Handler::AutoPage;
 # ABSTRACT: Class for handling the AutoPage feature
-$Dancer2::Handler::AutoPage::VERSION = '0.152000';
+$Dancer2::Handler::AutoPage::VERSION = '0.166001';
 use Moo;
 use Carp 'croak';
 use Dancer2::Core::Types;
 
-with 'Dancer2::Core::Role::Handler';
-with 'Dancer2::Core::Role::StandardResponses';
+with qw<
+    Dancer2::Core::Role::Handler
+    Dancer2::Core::Role::StandardResponses
+>;
 
 sub register {
     my ( $self, $app ) = @_;
@@ -25,17 +27,24 @@ sub code {
         my $app    = shift;
         my $prefix = shift;
 
-        my $page = $app->request->path_info;
-
-        my $template = $app->engine('template');
+        my $template = $app->template_engine;
         if ( !defined $template ) {
             $app->response->has_passed(1);
             return;
         }
 
+        my $page       = $app->request->path;
+        my $layout_dir = $template->layout_dir;
+        if ( $page =~ m{^/\Q$layout_dir\E/} ) {
+            $app->response->has_passed(1);
+            return;
+        }
+
+        # remove leading '/', ensuring paths relative to the view
+        $page =~ s{^/}{};
         my $view_path = $template->view_pathname($page);
 
-        if ( !-f $view_path ) {
+        if ( ! $template->pathname_exists( $view_path ) ) {
             $app->response->has_passed(1);
             return;
         }
@@ -64,7 +73,7 @@ Dancer2::Handler::AutoPage - Class for handling the AutoPage feature
 
 =head1 VERSION
 
-version 0.152000
+version 0.166001
 
 =head1 DESCRIPTION
 
@@ -111,7 +120,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
