@@ -1,6 +1,6 @@
 package Dancer2::Handler::File;
 # ABSTRACT: class for handling file content rendering
-$Dancer2::Handler::File::VERSION = '0.152000';
+$Dancer2::Handler::File::VERSION = '0.166001';
 use Carp 'croak';
 use Moo;
 use HTTP::Date;
@@ -9,17 +9,21 @@ use Dancer2::Core::MIME;
 use Dancer2::Core::Types;
 use File::Spec;
 
-with 'Dancer2::Core::Role::Handler';
-with 'Dancer2::Core::Role::StandardResponses';
-with 'Dancer2::Core::Role::Hookable';
+with qw<
+    Dancer2::Core::Role::Handler
+    Dancer2::Core::Role::StandardResponses
+    Dancer2::Core::Role::Hookable
+>;
 
-sub supported_hooks {
-    qw(
-      handler.file.before_render
-      handler.file.after_render
-    );
+sub hook_aliases {
+    {
+        before_file_render => 'handler.file.before_render',
+        after_file_render  => 'handler.file.after_render',
+    }
 }
 
+sub supported_hooks { values %{ shift->hook_aliases } }
+
 has mime => (
     is      => 'ro',
     isa     => InstanceOf ['Dancer2::Core::MIME'],
@@ -44,7 +48,7 @@ has regexp => (
 
 sub _build_public_dir {
     my $self = shift;
-    return $self->app->config->{public}
+    return $self->app->config->{public_dir}
         || $ENV{DANCER_PUBLIC}
         || path( $self->app->location, 'public' );
 }
@@ -73,21 +77,15 @@ sub code {
         my $path   = $app->request->path_info;
 
         if ( $path =~ /\0/ ) {
-            return $self->response_400($app);
+            return $self->standard_response( $app, 400 );
         }
 
         if ( $prefix && $prefix ne '/' ) {
             $path =~ s/^\Q$prefix\E//;
         }
 
-        my @tokens =
-          File::Spec->splitdir( join '',
-            ( File::Spec->splitpath($path) )[ 1, 2 ] );
-        if ( grep $_ eq '..', @tokens ) {
-            return $self->response_403($app);
-        }
-
-        my $file_path = path( $self->public_dir, @tokens );
+        my $file_path = $self->merge_paths( $path, $self->public_dir );
+        return $self->standard_response( $app, 403 ) if !defined $file_path;
 
         if ( !-f $file_path ) {
             $app->response->has_passed(1);
@@ -95,7 +93,7 @@ sub code {
         }
 
         if ( !-r $file_path ) {
-            return $self->response_403($app);
+            return $self->standard_response( $app, 403 );
         }
 
         # Now we are sure we can render the file...
@@ -133,6 +131,24 @@ sub code {
     };
 }
 
+sub merge_paths {
+    my ( undef, $path, $public_dir ) = @_;
+
+    my ( $volume, $dirs, $file ) = File::Spec->splitpath( $path );
+    my @tokens = File::Spec->splitdir( "$dirs$file" );
+    my $updir = File::Spec->updir;
+    return if grep $_ eq $updir, @tokens;
+
+    my ( $pub_vol, $pub_dirs, $pub_file ) = File::Spec->splitpath( $public_dir );
+    my @pub_tokens = File::Spec->splitdir( "$pub_dirs$pub_file" );
+    return if length $volume and length $pub_vol and $volume ne $pub_vol;
+
+    my @final_vol = ( length $pub_vol ? $pub_vol : length $volume ? $volume : () );
+    my @file_path = ( @final_vol, @pub_tokens, @tokens );
+    my $file_path = path( @file_path );
+    return $file_path;
+}
+
 1;
 
 __END__
@@ -147,7 +163,7 @@ Dancer2::Handler::File - class for handling file content rendering
 
 =head1 VERSION
 
-version 0.152000
+version 0.166001
 
 =head1 AUTHOR
 
@@ -155,7 +171,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
