@@ -1,13 +1,10 @@
-# ABSTRACT: Useful routines for testing Dancer2 apps
-
 package Dancer2::Test;
-{
-  $Dancer2::Test::VERSION = '0.11';
-}
+# ABSTRACT: Useful routines for testing Dancer2 apps
+$Dancer2::Test::VERSION = '0.166001';
 use strict;
 use warnings;
 
-use Carp 'croak';
+use Carp qw<carp croak>;
 use Test::More;
 use Test::Builder;
 use URI::Escape;
@@ -46,10 +43,11 @@ our @EXPORT = qw(
 use Dancer2::Core::Dispatcher;
 use Dancer2::Core::Request;
 
-
 # singleton to store all the apps
 my $_dispatcher = Dancer2::Core::Dispatcher->new;
 
+# prevent deprecation warnings
+our $NO_WARN = 0;
 
 # can be called with the ($method, $path, $option) triplet,
 # or can be fed a request object directly, or can be fed
@@ -57,6 +55,8 @@ my $_dispatcher = Dancer2::Core::Dispatcher->new;
 # or can be fed a response (which is passed through without
 # any modification)
 sub dancer_response {
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
 
     _find_dancer_apps_for_dispatcher();
 
@@ -68,9 +68,28 @@ sub dancer_response {
       ? _build_env_from_request(@_)
       : _build_request_from_env(@_);
 
-    return $_dispatcher->dispatch( $env, $request );
+    # override the set_request so it actually sets our request instead
+    {
+        no warnings qw<redefine once>;
+        *Dancer2::Core::App::set_request = sub {
+            my $self = shift;
+            $self->_set_request( $request );
+            $_->set_request( $request ) for @{ $self->defined_engines };
+        };
+    }
+
+    # since the response is a PSGI response
+    # we create a Response object which was originally expected
+    my $psgi_response = $_dispatcher->dispatch($env);
+    return Dancer2::Core::Response->new(
+        status  => $psgi_response->[0],
+        headers => $psgi_response->[1],
+        content => $psgi_response->[2][0],
+    );
 }
 
+
+
 sub _build_request_from_env {
 
     # arguments can be passed as the triplet
@@ -174,7 +193,7 @@ sub _build_env_from_request {
         SERVER_NAME       => 'localhost',
         SERVER_PORT       => 3000,
         HTTP_HOST         => 'localhost',
-        HTTP_USER_AGENT   => "Dancer2::Test simulator v $Dancer2::VERSION",
+        HTTP_USER_AGENT   => "Dancer2::Test simulator v" . Dancer2->VERSION,
     };
 
     # TODO
@@ -200,9 +219,10 @@ sub _build_env_from_request {
     return ( $request, $env );
 }
 
-
 sub response_status_is {
     my ( $req, $status, $test_name ) = @_;
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
 
     $test_name ||= "response status is $status for " . _req_label($req);
 
@@ -210,33 +230,59 @@ sub response_status_is {
 
     my $tb = Test::Builder->new;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
-    $tb->is_eq( $response->status, $status, $test_name );
+    $tb->is_eq( $response->[0], $status, $test_name );
 }
 
+sub _find_route_match {
+    my ( $request, $env ) =
+      ref $_[0] eq 'Dancer2::Core::Request'
+      ? _build_env_from_request(@_)
+      : _build_request_from_env(@_);
+
+    for my $app (@{$_dispatcher->apps}) {
+        for my $route (@{$app->routes->{lc($request->method)}}) {
+            if ( $route->match($request) ) {
+                return 1;
+            }
+        }
+    }
+    return 0;
+}
 
 sub route_exists {
-    response_status_is( $_[0], 200, $_[1] );
-}
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
 
+    my $tb = Test::Builder->new;
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    $tb->ok( _find_route_match($_[0]), $_[1]);
+}
 
 sub route_doesnt_exist {
-    response_status_is( $_[0], 404, $_[1] );
-}
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
 
+    my $tb = Test::Builder->new;
+    local $Test::Builder::Level = $Test::Builder::Level + 1;
+    $tb->ok( !_find_route_match($_[0]), $_[1]);
+}
 
 sub response_status_isnt {
     my ( $req, $status, $test_name ) = @_;
+
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
+
     $test_name ||= "response status is not $status for " . _req_label($req);
 
     my $response = dancer_response($req);
 
     my $tb = Test::Builder->new;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
-    $tb->isnt_eq( $response->status, $status, $test_name );
+    $tb->isnt_eq( $response->[0], $status, $test_name );
 }
 
 {
-
     # Map comparison operator names to human-friendly ones
     my %cmp_name = (
         is_eq   => "is",
@@ -259,37 +305,42 @@ sub response_status_isnt {
 
         my $tb = Test::Builder->new;
         local $Test::Builder::Level = $Test::Builder::Level + 1;
-        $tb->$cmp( $response->content, $want, $test_name );
+        $tb->$cmp( $response->[2][0], $want, $test_name );
     }
 }
 
-
 sub response_content_is {
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     _cmp_response_content( @_, 'is_eq' );
 }
 
-
 sub response_content_isnt {
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     _cmp_response_content( @_, 'isnt_eq' );
 }
 
-
 sub response_content_like {
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     _cmp_response_content( @_, 'like' );
 }
 
-
 sub response_content_unlike {
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     _cmp_response_content( @_, 'unlike' );
 }
 
-
 sub response_content_is_deeply {
     my ( $req, $matcher, $test_name ) = @_;
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     $test_name ||= "response content looks good for " . _req_label($req);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -297,9 +348,10 @@ sub response_content_is_deeply {
     is_deeply $response->[2][0], $matcher, $test_name;
 }
 
-
 sub response_is_file {
     my ( $req, $test_name ) = @_;
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     $test_name ||= "a file is returned for " . _req_label($req);
 
     my $response = _get_file_response($req);
@@ -308,23 +360,25 @@ sub response_is_file {
     return $tb->ok( defined($response), $test_name );
 }
 
-
 sub response_headers_are_deeply {
     my ( $req, $expected, $test_name ) = @_;
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     $test_name ||= "headers are as expected for " . _req_label($req);
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     my $response = dancer_response( _expand_req($req) );
 
     is_deeply(
-        _sort_headers( $response->headers_to_array ),
+        _sort_headers( $response->[1] ),
         _sort_headers($expected), $test_name
     );
 }
 
-
 sub response_headers_include {
     my ( $req, $expected, $test_name ) = @_;
+    carp 'Dancer2::Test is deprecated, please use Plack::Test instead'
+        unless $NO_WARN;
     $test_name ||= "headers include expected data for " . _req_label($req);
     my $tb = Test::Builder->new;
 
@@ -332,16 +386,15 @@ sub response_headers_include {
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
     print STDERR "Headers are: "
-      . Dumper( $response->headers_to_array )
+      . Dumper( $response->[1] )
       . "\n Expected to find header: "
       . Dumper($expected)
       if !$tb->ok(
-        _include_in_headers( $response->headers_to_array, $expected ),
+        _include_in_headers( $response->[1], $expected ),
         $test_name
       );
 }
 
-
 sub route_pod_coverage {
 
     require Pod::Simple::Search;
@@ -364,8 +417,11 @@ sub route_pod_coverage {
         $all_routes->{ $app->name }{routes} = [@$available_routes]
           if @$available_routes;
 
+        # Pod::Simple v3.30 excluded the current directory even when in @INC.
+        # include the current directory as a search path; its backwards compatible
+        # with previous version.
         my $undocumented_routes = [];
-        my $file                = Pod::Simple::Search->new->find( $app->name );
+        my $file                = Pod::Simple::Search->new->find( $app->name, '.' );
         if ($file) {
             $all_routes->{ $app->name }{has_pod} = 1;
             my $parser       = Pod::Simple::SimpleTree->new->parse_file($file);
@@ -420,7 +476,6 @@ sub route_pod_coverage {
     return $all_routes;
 }
 
-
 sub is_pod_covered {
     my ($test_name) = @_;
 
@@ -447,7 +502,6 @@ sub is_pod_covered {
     }
 }
 
-
 sub import {
     my ( $class, %options ) = @_;
 
@@ -563,17 +617,46 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Dancer2::Test - Useful routines for testing Dancer2 apps
 
 =head1 VERSION
 
-version 0.11
+version 0.166001
+
+=head1 SYNOPSIS
+
+    use Test::More;
+    use Plack::Test;
+    use HTTP::Request::Common; # install separately
+
+    use YourDancerApp;
+
+    my $app  = YourDancerApp->to_app;
+    my $test = Plack::Test->create($app);
+
+    my $res = $test->request( GET '/' );
+    is( $res->code, 200, '[GET /] Request successful' );
+    like( $res->content, qr/hello, world/, '[GET /] Correct content';
+
+    done_testing;
 
 =head1 DESCRIPTION
 
-provides useful routines to test Dancer2 apps.
+B<DEPRECATED>.  Please use L<Plack::Test> instead as shown in the SYNOPSIS!
+
+This module will warn for a while until we actually remove it. This is to
+provide enough time to fully remove it from your system.
+
+If you need to remove the warnings, for now, you can set:
+
+    $Dancer::Test::NO_WARN = 1;
+
+This module provides useful routines to test Dancer2 apps. They are, however,
+buggy and unnecessary. L<Plack::Test> is advised instead.
 
 $test_name is always optional.
 
@@ -641,7 +724,14 @@ one given.
 =head2 route_exists([$method, $path], $test_name)
 
 Asserts that the given request matches a route handler in Dancer2's
-registry.
+registry. If the route would have returned a 404, the route still exists
+and this test will pass.
+
+Note that because Dancer2 uses the default route handler
+L<Dancer2::Handler::File> to match files in the public folder when
+no other route matches, this test will always pass.
+You can disable the default route handlers in the configs but you probably
+want L<Dancer2::Test/response_status_is> or L<Dancer2::Test/dancer_response>
 
     route_exists [GET => '/'], "GET / is handled";
 
@@ -650,6 +740,10 @@ registry.
 Asserts that the given request does not match any route handler
 in Dancer2's registry.
 
+Note that this test is likely to always fail as any route not matched will
+be handled by the default route handler in L<Dancer2::Handler::File>.
+This can be disabled in the configs.
+
     route_doesnt_exist [GET => '/bogus_path'], "GET /bogus_path is not handled";
 
 =head2 response_status_isnt([$method, $path], $status, $test_name)
@@ -869,7 +963,7 @@ Dancer Core Developers
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2013 by Alexis Sukrieh.
+This software is copyright (c) 2015 by Alexis Sukrieh.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
